import re

import jstyleson

from ezregex import *
from ezregex import python


def test_replacement():
    offset = 2
    with open('tests/data/replacements.jsonc') as f:
        replacements = jstyleson.load(f)

    for pattern_str, repl_str, s, ans in replacements:
        try:
            pattern = eval(pattern_str, python.__dict__)
        except Exception as err:
            raise ValueError(f'Error compiling pattern {pattern_str}') from err

        try:
            repl = eval(repl_str, python.__dict__)
        except Exception as err:
            raise ValueError(f'Error compiling replacement pattern {repl_str}') from err

        assert re.sub(str(pattern), str(repl), s) == ans, \
            f'Replacing\n\t`{pattern}`\nwith\n\t`{repl}`\nin\n\t`{s}`\nyielded\n\t`{re.sub(str(pattern), str(repl), s)}`\nnot\n\t`{ans}`'

def test_replace_func():
    assert replace('|{g}|{1}|{0}|') == '|' + rgroup('g') + '|' + rgroup(1) + '|' + replace_entire + '|'
    assert replace("{group}this is am{group}mtest") == rgroup('group') + 'this is am' + rgroup('group') + "mtest"
    assert replace("this is {{ not a thing") == "this is { not a thing"
    assert replace("also not }} a thing") == "also not } a thing"
    assert replace("still }}not{{ a thing") == "still }not{ a thing"
    assert replace("also {{not}} a thing") == "also {not} a thing"
    assert replace("but {group} is and {1} is") == "but " + rgroup('group') + " is and " + rgroup(1) + " is"
    assert replace("{group}{g}") == rgroup('group') + rgroup('g')



import threading
import time

from rich import print as rprint
from rich.table import Table
from rich.text import Text

from ezregex import invert, python
from ezregex.python import literal

strictness=20
dontIncludePassed=True
invertBackend=...
invert_tries=5
import jstyleson


def test_invert():
    offset = 2

    # The structure is ["regex pattern", "replacement regex", "base string", "what the base string should look like after substitution"]
    with open('tests/data/replacements.jsonc') as f:
        replacements = jstyleson.load(f)

    with open('tests/data/regexs.jsonc') as f:
        regexs = jstyleson.load(f)

    # with open('deleteme.txt', 'w') as f:
    #     rprint(regexs + replacements, file=f)

    table = Table(title="invert tests", expand=False)
    table.add_column("Line", justify="center", style="grey37")#, max_width=2)
    table.add_column("Regex", justify="right", style="green")#, max_width=40)
    table.add_column("Inverse", justify="left", style="dim green")
    table.add_column("Success", justify="center")

    for cnt, r in enumerate(regexs + replacements):
        def do_test():
            # with open('log.txt', 'a') as f:
            #     rprint(f'testing {r[0]}', file=f)
            try:
                regex = eval(r[0], python.__dict__)

                if type(regex) is str:
                    regex = literal(regex)
                for _ in range(strictness):
                    # -1 means return it even if it's bad
                    inv = invert(regex, backend=invertBackend, tries=invert_tries)
                    if inv not in regex or not dontIncludePassed:
                        table.add_row(str(offset+(cnt*5)), Text(regex.str()), '`' + inv + '`', Text('passed', style='blue') if inv in regex else Text('failed', style='red'))
            except (Exception, AssertionError) as err:
                raise ValueError(f'Error @ approx. {__file__}, line {offset+(cnt*4)}: \nregex = `{r[0]}`') #from err#, inv = `{inv}`')
        thread = threading.Thread(target=do_test)
        thread.start()
        limit = 5
        thread.join(timeout=limit)
        assert not thread.is_alive(), f"Invert took too long (>{limit} seconds): `{r[0]}`'"

    if len(table.rows):
        with open('summary.txt', 'w') as f:
            rprint(table, file=f)
    assert not len(table.rows)



from ..EZRegex import EZRegex


class PerlEZRegex(EZRegex):
    _escape_chars=b'()[]{}?*+-|^$\\.&~# \t\n\r\v\f'
    _end = ''
    _beginning = ''

    def _flag_func(self, final):
        return f'(?{self.flags}){final}'
